import torch
from torch import nn
from torch.utils.data import Dataset
from torchvision import transforms
import os
import timm
from PIL import Image

class CFG:
    mean = [0.6289, 0.6249, 0.5952]
    std = [0.2096, 0.1951, 0.1963]
    height = 224
    width = 336
    num_classes = 4
    stoi = {'blur' : 0, 'missing' : 1, 'overexposed' : 2, 'sharp' : 3}
    itos = {0 :'blur', 1 : 'missing', 2 : 'overexposed', 3 : 'sharp'}
    target_names = ['blur', 'missing', 'overexposed', 'sharp']

class IDCardQuailityClassification:
    def __init__(self) -> None:
        device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
        self.model = timm.create_model("efficientnet_b4", pretrained=True)
        for param in self.model.parameters():
            param.requires_grad = True
        self.model.classifier = nn.Linear(self.model.classifier.in_features, CFG.num_classes)
        self.model.to(device)
        self.model.load_state_dict(
            torch.load(
                os.path.join(
                    os.path.dirname(os.path.realpath(__file__)),
                    'non-overfit_model.pt'
                ), 
                map_location=torch.device('cpu')
            )
        )
    def get_transforms(self):
        return transforms.Compose([
            transforms.Resize((CFG.height,CFG.width)),
            transforms.ToTensor(),
            transforms.Normalize(mean=CFG.mean,std=CFG.std)
        ])
    def compare(self,img_path, label):
        ##########################################
        # input: path to image, label of image
        # output: prediction == label
        ##########################################

        self.model.eval()
        img = Image.open(img_path).convert("RGB")
        tensor_img = self.get_transforms()(img).unsqueeze(0)
        outputs = self.model(tensor_img)
        _, pred = torch.topk(outputs, 2, 1)
        return CFG.stoi[label] in pred, pred